@@ -16,6 +16,8 @@
 #ifndef FPTOOLS_H__
 #define FPTOOLS_H__
 
+#include <string.h>
+
 typedef signed char schar;
 typedef unsigned char uchar;
 
@@ -27,6 +29,22 @@ typedef unsigned char uchar;
 extern "C" {
 #endif
 
+/* relicensed from libecb with permission */
+#if 1
+
+#define ECB_GCC_VERSION(major,minor) (__GNUC__ > (major) || (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
+
+#if ECB_GCC_VERSION(3,1)
+  #define ecb_expect(expr,value)         __builtin_expect ((expr),(value))
+#else
+  #define ecb_expect(expr,value)         (expr)
+#endif
+
+#define ecb_expect_false(expr) ecb_expect (!!(expr), 0)
+#define ecb_expect_true(expr)  ecb_expect (!!(expr), 1)
+
+#endif
+
 #define _FP_free	FP_free
 #define _FP_strdup	FP_strdup
 #define _FP_strncpy	FP_strncpy
@@ -47,18 +65,26 @@ extern "C" {
 #define _FP_cutdir	FP_cutdir
 #define _FP_strerror	FP_strerror
 #define _FP_tempnam	FP_tempnam
+#define _FP_fgetc	FP_fgetc
+
+#if HAVE_FGETC_UNLOCKED
+# define FP_fgetc(s) fgetc_unlocked (s)
+#else
+# define FP_fgetc(s) fgetc (s)
+#endif
+
+#define FP_strstr(a,b) strstr (a, b)
 
 void	TOOLEXPORT	_FP_free	(void *);
 char *	TOOLEXPORT	_FP_strdup	(char *);
 char *	TOOLEXPORT	_FP_strncpy	(char *, char *, int);
 void *	TOOLEXPORT	_FP_memdup	(void *, int);
-int 	TOOLEXPORT	_FP_stricmp	(char *, char *);
-int 	TOOLEXPORT	_FP_strnicmp	(char *, char *, int);
+int	TOOLEXPORT	_FP_stricmp	(const char *, const char *);
+int	TOOLEXPORT	_FP_strnicmp	(const char *, const char *, int);
 char *	TOOLEXPORT	_FP_strrstr	(char *, char *);
 char *	TOOLEXPORT	_FP_stoupper	(char *);
 char *	TOOLEXPORT	_FP_stolower	(char *);
-int 	TOOLEXPORT	_FP_strmatch	(char *, char *);
-char *	TOOLEXPORT	_FP_strstr	(char *, char *);
+int	TOOLEXPORT	_FP_strmatch	(char *, char *);
 char *	TOOLEXPORT	_FP_stristr	(char *, char *);
 char *	TOOLEXPORT	_FP_strirstr	(char *, char *);
 char *	TOOLEXPORT	_FP_strrchr	(char *, int);
@@ -71,6 +97,20 @@ char *	TOOLEXPORT	_FP_strerror	(int);
 char *	TOOLEXPORT	_FP_tempnam	(char *, char *);
 #endif /* HAVE_MKSTEMP */
 
+#if 0 /* API differs too much between systems to make those replacements */
+#if HAVE_STRCASECMP
+# define FP_stricmp(a,b) strcasecmp (a, b)
+#endif
+
+#if HAVE_STRNCASECMP
+# define FP_strnicmp(a,b,l) strncasecmp (a, b, l)
+#endif
+
+#if HAVE_STRCASESTR
+# define FP_stristr(a,b) strcasestr (a, b)
+#endif
+#endif
+
 #ifdef __cplusplus
 }
 #endif
